package net.eusashead.hateoas.hal.response.impl;

/*
 * #[license]
 * spring-halbuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

import javax.servlet.http.HttpServletRequest;

import org.springframework.http.ResponseEntity;

import net.eusashead.hateoas.header.ETagHeaderStrategy;
import net.eusashead.hateoas.header.impl.HashingETagHeaderStrategy;
import net.eusashead.hateoas.response.ResponseBuilder;
import net.eusashead.hateoas.response.impl.AbstractResponseBuilder;

import com.theoryinpractise.halbuilder.api.Representation;
import com.theoryinpractise.halbuilder.api.RepresentationFactory;

/**
 * Abstract class from which HAL {@link Representation}
 * {@link ResponseEntity} can be constructed using
 * HALBuilder
 * 
 * @author patrickvk
 *
 */
public abstract class AbstractHalGetResponseBuilder extends AbstractResponseBuilder<Representation> {

	/**
	 * RepresentationFactor to create HAL Representations
	 */
	protected final RepresentationFactory representationFactory;
	
	/**
	 * URI of the response entity
	 */
	protected final String uri;
	
	/**
	 * HAL Representation wrapped by this response
	 */
	protected Representation representation;

	/**
	 * Construct a {@link ResponseBuilder} with
	 * the supplied {@link RepresentationFactory}
	 * and {@link HttpServletRequest}
	 * @param representationFactory {@link RepresentationFactory} for constructing the HAL {@link Representation}
	 * @param request incoming {@link HttpServletRequest} for which the response is built for
	 */
	public AbstractHalGetResponseBuilder(RepresentationFactory representationFactory, HttpServletRequest request) {

		super(request);

		// Set the RepresentationFactory
		this.representationFactory = representationFactory;

		// Create the URI for the resource
		StringBuilder uriBuilder = new StringBuilder(request.getRequestURI());
		if (request.getQueryString() != null) {
			uriBuilder.append("?");
			uriBuilder.append(request.getQueryString());
		}
		this.uri = uriBuilder.toString();
		
		// Create representation
		this.representation = representationFactory.newRepresentation(uri);
	}

	/**
	 * Set the ETag header using the default {@link HashingETagHeaderStrategy}
	 */
	protected void setEtagHeader() {
		assertRepresentation();
		this.headers.setETag(new HashingETagHeaderStrategy().extract(representation).toString()); 
	}

	/**
	 * Set the ETag header using the supplied {@link ETagHeaderStrategy}
	 * @param etagStrategy {@link ETagHeaderStrategy} to use to extract an ETag from the HAL {@link Representation}
	 */
	protected void setEtagHeader(ETagHeaderStrategy etagStrategy) {
		assertRepresentation();
		this.headers.setETag(etagStrategy.extract(representation).toString()); 
	}

	/**
	 * Override the wrapped {@link Representation}  
	 * the supplied {@link Representation}
	 * @param representation {@link Representation} to set in the response
	 */
	protected void setRepresentation(Representation representation) {
		this.representation = representation;
		assertRepresentation();
	}

	/**
	 * Assert that the {@link Representation} has
	 * been set or constructed. Called before using
	 * an {@link ETagHeaderStrategy} or building the 
	 * {@link ResponseEntity} to ensure that it is not
	 * null.
	 */
	protected void assertRepresentation() {
		if (representation == null) {
			throw new IllegalArgumentException("Representation cannot be null.");
		}
	}

}
