package net.eusashead.hateoas.hal.response.impl;

/*
 * #[license]
 * spring-halbuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

import java.util.Date;

import javax.servlet.http.HttpServletRequest;

import net.eusashead.hateoas.hal.adapter.RepresentationWriter;
import net.eusashead.hateoas.hal.response.HalGetResponseBuilder;
import net.eusashead.hateoas.header.ETagHeaderStrategy;
import net.eusashead.hateoas.response.ResponseBuilder;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import com.theoryinpractise.halbuilder.api.ReadableRepresentation;
import com.theoryinpractise.halbuilder.api.Representation;
import com.theoryinpractise.halbuilder.api.RepresentationFactory;

/**
 * {@link ResponseBuilder} that enables
 * a HAL {@link Representation} to be
 * constructed for the response.
 * 
 * It exposes a similar API to the
 * HALBuilder {@link Representation} 
 * but automates the URI for the "self"
 * link to be that of the incoming request
 * (which is probably what you would want).
 * 
 * It also contains the methods for adding
 * ETag, Last-Modified, Date, Cache-Control
 * and Expires headers to help with caching
 * and bandwidth conservation.
 * 
 * @author patrickvk
 *
 */
public class HalGetResponseBuilderImpl extends AbstractHalGetResponseBuilder implements HalGetResponseBuilder {

	public HalGetResponseBuilderImpl(RepresentationFactory representationFactory, HttpServletRequest request) {
		super(representationFactory, request);
	}
	
	@Override
	public HalGetResponseBuilderImpl withProperty(String name, Object value) {
		assertRepresentation();
		this.representation.withProperty(name, value);
		return this;
	}
	
	@Override
	public HalGetResponseBuilderImpl withRepresentation(String rel, ReadableRepresentation resource) {
		assertRepresentation();
		this.representation.withRepresentation(rel, resource);
		return this;
	}
	
	@Override
	public HalGetResponseBuilderImpl withLink(String rel, String href) {
		assertRepresentation();
		this.representation.withLink(rel, href);
		return this;
	}
	
	@Override
	public HalGetResponseBuilderImpl withNamespace(String namespace, String href) {
		assertRepresentation();
		this.representation.withNamespace(namespace, href);
		return this;
	}
	
	@Override
	public HalGetResponseBuilderImpl withBean(Object value) {
		assertRepresentation();
		this.representation.withBean(value);
		return this;
	}
	
	@Override
	public HalGetResponseBuilderImpl withBeanBasedRepresentation(String rel, String href, Object o) {
		assertRepresentation();
		this.representation.withBeanBasedRepresentation(rel, href, o);
		return this;
	}
	
	@Override
	public HalGetResponseBuilderImpl representation(Representation representation) {
		super.setRepresentation(representation);
		return this;
	}
	
	@Override
	public HalGetResponseBuilderImpl convert(Object bean) {
		assertRepresentation();
		this.representation.withBean(bean);
		return this;
	}
	
	@Override
	public <T> HalGetResponseBuilderImpl convert(T bean, RepresentationWriter<T> writer) {
		this.representation = writer.write(bean, representationFactory);
		return this;
	}

	@Override
	public HalGetResponseBuilderImpl etag() {
		super.setEtagHeader();
		return this;
	}

	@Override
	public HalGetResponseBuilderImpl etag(ETagHeaderStrategy strategy) {
		super.setEtagHeader(strategy);
		return this;
	}

	@Override
	public HalGetResponseBuilderImpl etag(Date date) {
		super.setEtagHeader(date);
		return this;
	}

	@Override
	public HalGetResponseBuilderImpl etag(Integer version) {
		super.setEtagHeader(version);
		return this;
	}

	@Override
	public HalGetResponseBuilderImpl lastModified(Date date) {
		super.setLastModifiedHeader(date);
		return this;
	}

	@Override
	public HalGetResponseBuilderImpl expireIn(long millis) {
		super.setExpiryHeaders(millis);
		return this;
	}

	@Override
	public ResponseEntity<Representation> build() {
		super.assertRepresentation();
		
		// ResponseEntity to return
		ResponseEntity<Representation> responseEntity;
		
		// Check ETag against request value
		if (headers.getETag() != null && compareEtagWithIfNoneMatch(headers.getETag())) {
			// Return 304 Not Modified
			responseEntity = new ResponseEntity<Representation>(headers, HttpStatus.NOT_MODIFIED);
		} else {
			// Return 200 and the entity
			responseEntity = new ResponseEntity<Representation>(representation, headers, HttpStatus.OK);
		}
		
		// Return the ResponseEntity
		return responseEntity;
	}

}
